import argparse
import json
import numpy as np
import matplotlib.pyplot as plt

from danmaku_energy_map import read_danmaku_file, draw_he_line, get_heat_time, draw_he_annotate, \
    convert_time, get_value

parser = argparse.ArgumentParser(description='Get gift analytics for BiliBili Live XML')
parser.add_argument('danmaku', type=str, help='path to the danmaku file')
# parser.add_argument('--graph', type=str, default=None, help='output graph path, leave empty if not needed')
# parser.add_argument('--he_map', type=str, default=None, help='output high density timestamp, leave empty if not needed')
# parser.add_argument('--sc_list', type=str, default=None, help='output super chats, leave empty if not needed')


if __name__ == '__main__':
    args = parser.parse_args()
    xml_list = read_danmaku_file(args.danmaku, guard=True)
    total_d = 0
    total_sc = 0
    total_gift = 0
    total_guard = 0
    guard_map = {}

    for item in xml_list:
        if item.tag == 'd':
            total_d += 1
        elif item.tag == 'sc':
            total_sc += get_value(item) * 10
        elif item.tag == 'gift':
            total_gift += get_value(item) * 10
        elif item.tag == 'guard':
            total_guard += get_value(item) * 10
            raw_data = json.loads(item.attrib['raw'])
            gift_name = raw_data["gift_name"]
            if gift_name in guard_map:
                guard_map[gift_name] += 1
            else:
                guard_map[gift_name] = 1

    print(f"弹幕：{total_d}条")
    print(f"醒目留言：{total_sc}元")
    print(f"礼物：{total_gift}元")
    print(f"大航海：{total_guard}元")
    print(f"大航海类别：{guard_map}")



    # if args.graph is not None:
    #     heat_time, heat_value_gaussian, heat_value_gaussian2, he_points = get_heat_time(xml_list)
    #
    #     fig = plt.figure(figsize=(heat_time[0][-1] / 1000, 4))
    #
    #     draw_he_line(fig.gca(), heat_time, heat_value_gaussian, heat_value_gaussian2)
    #
    #     for name, name_chn in {'d': "danmaku", 'sc': "superchat", 'gift': "gift", 'guard': 'guard'}.items():
    #         part_xml_list = [element for element in xml_list if element.tag == name]
    #         heat_time, heat_value_gaussian, heat_value_gaussian2, _ = get_heat_time(part_xml_list)
    #         if name in ['sc', 'gift']:
    #             draw_he_line(fig.gca(), heat_time, heat_value_gaussian * 10, heat_value_gaussian2 * 10, name=name_chn)
    #
    #     part_xml_list = [element for element in xml_list if element.tag in ['sc', 'gift']]
    #     heat_time, heat_value_gaussian, heat_value_gaussian2, he_points = get_heat_time(part_xml_list)
    #     draw_he_annotate(fig.gca(), heat_time, he_points)
    #
    #     t_x = heat_time[0][::1000]
    #
    #     t = [convert_time(time) for time in t_x]
    #
    #     plt.xticks(t_x, t)
    #
    #     plt.legend()
    #
    #     plt.savefig(args.graph)



import argparse
import subprocess
import xml.etree.ElementTree as ET
from dateutil.parser import parse

parser = argparse.ArgumentParser(description='Merge BiliLiveReocrder XML')
parser.add_argument('xml_files', type=str, nargs='+', help='path to the danmaku file')
parser.add_argument('--video_time', type=str, default="", help='use video length as the duration of the file')
parser.add_argument('--output', type=str, default=None, help='output path for the output XML', required=True)


def get_root_time(root_xml):
    record_info = root_xml.findall('BililiveRecorderRecordInfo')[0]
    record_start_time_str = record_info.attrib['start_time']
    record_start_time = parse(record_start_time_str)
    return record_start_time


def add_root(orig_root, new_root, new_offset):
    for child in new_root:
        if child.tag in ['sc', 'gift', 'guard']:
            orig_time = float(child.attrib['ts'])
            new_time = orig_time + new_offset
            new_time_str = str(new_time)
            child.set('ts', new_time_str)
            orig_root.append(child)
        if child.tag in ['d']:
            orig_parameters_str = child.attrib['p'].split(',')
            orig_time = float(orig_parameters_str[0])
            new_time = orig_time + new_offset
            new_parameters_str = [str(new_time)] + orig_parameters_str[1:]
            child.set('p', ','.join(new_parameters_str))
            orig_root.append(child)


if __name__ == '__main__':
    args = parser.parse_args()

    if len(args.xml_files) == 0:
        print("At least one XML files have to be passed as input.")

    tree = ET.parse(args.xml_files[0])
    root = tree.getroot()
    new_root_offset = 0
    root_time = get_root_time(root)
    all_flv = ""

    for i in range(len(args.xml_files) - 1):
        new_root = ET.parse(args.xml_files[i + 1]).getroot()
        if args.video_time == "":
            new_root_offset = (get_root_time(new_root) - root_time).total_seconds()
        else:
            prev_xml_path = args.xml_files[i]
            base_file_path = prev_xml_path.rpartition('.')[0]
            flv_file_path = base_file_path + args.video_time
            total_seconds_str = subprocess.check_output(
                f'ffprobe -v error -show_entries format=duration '
                f'-of default=noprint_wrappers=1:nokey=1 "{flv_file_path}"', shell=True
            )
            all_flv += flv_file_path + "\n"
            new_root_offset += float(total_seconds_str)
        add_root(root, new_root, new_root_offset)

    tree.write(args.output, encoding='UTF-8', xml_declaration=True)



import argparse
import xml.etree.ElementTree as ET

from dateutil.parser import parse

parser = argparse.ArgumentParser(description='Merge BiliLiveReocrder XML')
parser.add_argument('xml_files', type=str, help='path to the danmaku file')
parser.add_argument('--start_time', type=float, help='use video length as the duration of the file', default=0)
parser.add_argument('--end_time', type=float, help='use video length as the duration of the file', default=float('inf'))
parser.add_argument('--output', type=str, default=None, help='output path for the output XML', required=True)


def get_root_time(root_xml):
    record_info = root_xml.findall('BililiveRecorderRecordInfo')[0]
    record_start_time_str = record_info.attrib['start_time']
    record_start_time = parse(record_start_time_str)
    return record_start_time


def process_root(orig_root, start_time, end_time):
    new_root = ET.Element('i')
    for child in orig_root:
        if child.tag in ['sc', 'gift', 'guard']:
            orig_time = float(child.attrib['ts'])
            if start_time <= orig_time <= end_time:
                new_time = orig_time - start_time
                new_time_str = str(new_time)
                child.set('ts', new_time_str)
                new_root.append(child)
        elif child.tag in ['d']:
            orig_parameters_str = child.attrib['p'].split(',')
            orig_time = float(orig_parameters_str[0])
            if start_time <= orig_time <= end_time:
                new_time = orig_time - start_time
                new_parameters_str = [str(new_time)] + orig_parameters_str[1:]
                child.set('p', ','.join(new_parameters_str))
                new_root.append(child)
        else:
            new_root.append(child)
    return new_root


if __name__ == '__main__':
    args = parser.parse_args()

    tree = ET.parse(args.xml_files)
    root = tree.getroot()
    new_root_offset = 0
    root_time = get_root_time(root)
    all_flv = ""

    new_root = process_root(root, args.start_time, args.end_time)

    ET.ElementTree(new_root).write(args.output, encoding='UTF-8', xml_declaration=True)
